@@ -127,27 +127,51 @@ class GetS3Handler(AWSConfig):
         """
         
         return self._list_objects(bucket_name,obj_path,include_filters,exclude_filters)
-        
+    
     def upload_objects(self,
                      bucket_name: str,
-                     s3_obj_path: str,
-                     local_obj_path: str):
+                     s3_path: str,
+                     local_path: str,
+                     include_filters: list = [],
+                     exclude_filters: list = [],
+                     dryrun: bool = False):
         """Upload files/folders to s3 bucket
         
-        Args:
-            bucket_name (string): Name of bucket
-            s3_obj_path (string): Path on s3 bucket
-            local_obj_path (string): Local path on your machine
-        Returns:
-            string: output/error string
+        Args: \n
+            bucket_name (string): Name of bucket \n
+            s3_path (string): Path on s3 bucket \n
+            local_path (string): Local path on your machine \n
+            include_filters (list): list of sub strings to include in filtering  \n
+                                    Eg: ["20220101",".tif"] (No Wild card character allowed, only sub-strings) \n
+            exclude_filters (list): list of sub strings to exclude in filtering \n
+                                    Eg: ["20220101",".tif"] (No Wild card character allowed, only sub-strings) \n
+            dryrun (bool): Displays the operations that would be performed using the specified command without actually running them \n
+        Returns: \n
+            string: output/error string \n
         """
         
-        s3_obj_path = s3_obj_path.strip("/")
+        s3_path = s3_path.strip("/")
+        
+        include_filter_pattern = ""
+        exclude_filter_pattern = ""
+        dryrun_pattern = ""
+        
+        if len(include_filters) > 0:
+            include_filter_pattern +=  " ".join( [f'--include "*{include_filter}*"' for include_filter in include_filters ] )
 
-        if os.path.isdir(local_obj_path):
-            command = f"""aws s3 sync "{local_obj_path}" "s3://{bucket_name}/{s3_obj_path}" --request-payer "requester" --profile {self.AWS_PROFILE_NAME} --output json"""
+            include_filter_pattern = '--exclude "*" ' + include_filter_pattern
+            
+            
+        if len(exclude_filters) > 0: 
+            exclude_filter_pattern +=  " ".join( [f'--exclude "*{exclude_filter}*"' for exclude_filter in exclude_filters ] )
+            
+        if dryrun:
+            dryrun_pattern = f"--dryrun"
+        
+        if os.path.isdir(local_path):
+            command = f"""aws s3 cp "{local_path}" "s3://{bucket_name}/{s3_path}" {exclude_filter_pattern} {include_filter_pattern} {dryrun_pattern} --recursive --request-payer "requester" --profile {self.AWS_PROFILE_NAME} --output json"""
         else:
-            command = f"""aws s3 cp "{local_obj_path}" "s3://{bucket_name}/{s3_obj_path}" --request-payer "requester" --profile {self.AWS_PROFILE_NAME} --output json"""
+            command = f"""aws s3 cp "{local_path}" "s3://{bucket_name}/{s3_path}" {exclude_filter_pattern} {include_filter_pattern} {dryrun_pattern} --request-payer "requester" --profile {self.AWS_PROFILE_NAME} --output json"""
         
         process = Popen(command,shell=True,stdout=PIPE)
         stdout, _ = process.communicate()
@@ -155,10 +179,11 @@ class GetS3Handler(AWSConfig):
     
     def _download_objects(self,
                          bucket_name: str,
-                         s3_obj_path: str ,
-                         local_obj_path: str,
+                         s3_path: str ,
+                         local_path: str,
+                         bulk: bool,
                          include_filters: list = [],
-                         exclude_filters:list = [],
+                         exclude_filters: list = [],
                          dryrun: bool = False):
         
         include_filter_pattern = ""
@@ -167,27 +192,40 @@ class GetS3Handler(AWSConfig):
         
         if len(include_filters) > 0:
             include_filter_pattern +=  " ".join( [f'--include "*{include_filter}*"' for include_filter in include_filters ] )
-        
+
+            include_filter_pattern = '--exclude "*" ' + include_filter_pattern
+            
+            
         if len(exclude_filters) > 0: 
             exclude_filter_pattern +=  " ".join( [f'--exclude "*{exclude_filter}*"' for exclude_filter in exclude_filters ] )
             
         if dryrun:
             dryrun_pattern = f"--dryrun"
+            
+        if not os.path.exists(local_path):
+            os.makedirs(local_path)
+            
         
-        if os.path.isdir(local_obj_path):
-            command = f"""aws s3 sync {dryrun_pattern} "s3://{bucket_name}/{s3_obj_path}" "{local_obj_path}" {exclude_filter_pattern} {include_filter_pattern} --request-payer "requester" --profile {self.AWS_PROFILE_NAME} """
+        if bulk:
+            command = f"""aws s3 cp {dryrun_pattern} "s3://{bucket_name}/{s3_path}" "{local_path}" {exclude_filter_pattern} {include_filter_pattern} --recursive --request-payer "requester" --profile {self.AWS_PROFILE_NAME} """
         else:
-            command = f"""aws s3 cp {dryrun_pattern} "s3://{bucket_name}/{s3_obj_path}" "{local_obj_path}" {exclude_filter_pattern} {include_filter_pattern} --request-payer "requester" --profile {self.AWS_PROFILE_NAME} """
+            command = f"""aws s3 cp {dryrun_pattern} "s3://{bucket_name}/{s3_path}" "{local_path}" {exclude_filter_pattern} {include_filter_pattern} --request-payer "requester" --profile {self.AWS_PROFILE_NAME} """
         
+        print(command)
+        
+        try:
+            process = Popen(command,shell=True,stdout=PIPE)
+            stdout, stderr = process.communicate()
+            return stdout.decode("utf-8")
+        except Exception as E:
+            print("No output found")
+            return []
         
-        process = Popen(command,shell=True,stdout=PIPE)
-        stdout, _ = process.communicate()
-        return stdout.decode("utf-8")
-    
     def download_objects(self,
                          bucket_name: str,
-                         s3_obj_path: str ,
-                         local_obj_path: str=".",
+                         s3_path: str ,
+                         local_path: str=".",
+                         bulk: bool = False,
                          include_filters: list = [],
                          exclude_filters:list = [],
                          dryrun: bool = False):
@@ -195,8 +233,9 @@ class GetS3Handler(AWSConfig):
         
         Args:
             bucket_name (string): Name of bucket
-            s3_obj_path (string): path on s3 bucket
-            local_obj_path (string): Path on your machine
+            s3_path (string): path on s3 bucket
+            local_path (string): Path on your machine
+            bulk (bool): This allows to download files in bulk from bucket
             include_filters (list): list of sub strings to include in filtering 
                                     Eg: ["20220101",".tif"] (No Wild card character allowed, only sub-strings)
             exclude_filters (list): list of sub strings to exclude in filtering
@@ -207,8 +246,9 @@ class GetS3Handler(AWSConfig):
         """
         
         return self._download_objects(bucket_name,
-                               s3_obj_path,
-                               local_obj_path,
+                               s3_path,
+                               local_path,
+                               bulk,
                                include_filters,
                                exclude_filters,
                                dryrun)
\ No newline at end of file
